@@ -3,6 +3,7 @@ Loading data from audio files.
 
 - `load_audio()` loads a whole audio file at once.
 - `AudioLoader` allow to read data from audio files in chunks.
+- `blocks()`: generator for blockwise processing of array data.
 - `unwrap()` unwraps clipped data that are folded into the available data range.
 
 The read in data are always numpy arrays of floats ranging between -1 and 1.
@@ -26,8 +27,7 @@ from .audiomodules import *
 
 
 def load_wave(filepath, verbose=0):
-    """
-    Load wav file using the wave module from pythons standard libray.
+    """Load wav file using the wave module from pythons standard libray.
     
     Documentation
     -------------
@@ -83,8 +83,7 @@ def load_wave(filepath, verbose=0):
 
     
 def load_ewave(filepath, verbose=0):
-    """
-    Load wav file using ewave module.
+    """Load wav file using ewave module.
 
     Documentation
     -------------
@@ -126,8 +125,7 @@ def load_ewave(filepath, verbose=0):
 
     
 def load_wavfile(filepath, verbose=0):
-    """
-    Load wav file using scipy.io.wavfile.
+    """Load wav file using scipy.io.wavfile.
 
     Documentation
     -------------
@@ -175,8 +173,7 @@ def load_wavfile(filepath, verbose=0):
 
 
 def load_soundfile(filepath, verbose=0):
-    """
-    Load audio file using SoundFile (based on libsndfile).
+    """Load audio file using SoundFile (based on libsndfile).
 
     Documentation
     -------------
@@ -216,8 +213,7 @@ def load_soundfile(filepath, verbose=0):
 
 
 def load_wavefile(filepath, verbose=0):
-    """
-    Load audio file using wavefile (based on libsndfile).
+    """Load audio file using wavefile (based on libsndfile).
 
     Documentation
     -------------
@@ -252,8 +248,7 @@ def load_wavefile(filepath, verbose=0):
 
 
 def load_audioread(filepath, verbose=0):
-    """
-    Load audio file using audioread.
+    """Load audio file using audioread.
 
     Documentation
     -------------
@@ -293,13 +288,12 @@ def load_audioread(filepath, verbose=0):
         for buffer in af:
             fulldata = np.frombuffer(buffer, dtype='<i2').reshape(-1, af.channels)
             n = fulldata.shape[0]
-            if index+n > len(data):
-                n = len(data) - index
-            if n > 0:
-                data[index:index+n,:] = fulldata[:n,:]
-                index += n
-            else:
+            if index + n > len(data):
+                n = len(fulldata) - index
+            if n <= 0:
                 break
+            data[index:index+n,:] = fulldata[:n,:]
+            index += n
     return data/(2.0**15-1.0), float(rate)
 
 
@@ -311,15 +305,14 @@ audio_loader_funcs = (
     ('ewave', load_ewave),
     ('scipy.io.wavfile', load_wavfile)
     )
-""" List of implemented load functions.
+"""List of implemented load functions.
 
 Each element of the list is a tuple with the module's name and the load function.
 """    
 
 
 def load_audio(filepath, verbose=0):
-    """
-    Call this function to load all channels of audio data from a file.
+    """Call this function to load all channels of audio data from a file.
     
     This function tries different python modules to load the audio file.
 
@@ -403,9 +396,71 @@ def load_audio(filepath, verbose=0):
     return data, rate
 
 
-def unwrap(data):
+def blocks(data, block_size, noverlap=0):
+    """Generator for blockwise processing of array data.
+
+    Parameters
+    ----------
+    data: array
+        Data to loop over. First dimension is time.
+    block_size: int
+        Len of data blocks to be returned.
+    noverlap: int
+        Number of indices successive data points should overlap.
+
+    Yields
+    ------
+    data: array
+        Successive slices of the input data.
+
+    Raises
+    ------
+    ValueError
+        `noverlap` larger or equal to `block_size`.
+
+    Examples
+    --------
+    ```
+    import numpy as np
+    from audioio import blocks
+    data = np.arange(20)
+    for x in blocks(data, 6, 2):
+        print(x)
+    ```
+    results in
+    ```text
+    [0 1 2 3 4 5]
+    [4 5 6 7 8 9]
+    [ 8  9 10 11 12 13]
+    [12 13 14 15 16 17]
+    [16 17 18 19]
+    ```
+
+    Use it for processing long audio data, like computing a spectrogram with overlap:
+    ```
+    from scipy.signal import spectrogram
+    from audioio import AudioLoader, blocks
+    nfft = 2048
+    with AudioLoader('some/audio.wav') as data:
+        for x in blocks(data, 100*nfft, nfft//2):
+            f, t, Sxx = spectrogram(x, nperseg=nfft, noverlap=nfft//2)
+    ```
     """
-    Fixes data that exceeded the -1 to 1 range.
+    if noverlap >= block_size:
+        raise ValueError('noverlap=%d larger than block_size=%d' % (noverlap, block_size))
+    m = block_size - noverlap
+    n = (len(data)-noverlap)//m
+    if n == 0:
+        yield data[:]
+    else:
+        for k in range(n):
+            yield data[k*m:k*m+block_size]
+        if len(data) - (k*m+block_size) > 0:
+            yield data[(k+1)*m:]
+
+
+def unwrap(data):
+    """Fixes data that exceeded the -1 to 1 range.
 
     If data that exceed the range from -1.0 to 1.0 are stored in a wav file,
     they get wrapped around. This functions tries to undo this wrapping.
@@ -449,16 +504,19 @@ class AudioLoader(object):
     audio modules until it succeeds (first line). It then reads data from the file
     as necessary for the requested data (second line).
 
-    Reading sequentially through the file is always possible. Some modules, however,
-    (e.g. audioread, needed for mp3 files) can only read forward. If previous data
-    are requested, then the file is read from the beginning. This slows down access
-    to previous data considerably. Use the backsize argument to the open functions to
-    make sure some data are loaded before the requested frame. Then a subsequent access
-    to the data within backsize seconds before that frame can still be handled without
-    the need to reread the file from the beginning.
+    Reading sequentially through the file is always possible. Some
+    modules, however, (e.g. audioread, needed for mp3 files) can only
+    read forward. If previous data are requested, then the file is read
+    from the beginning. This slows down access to previous data
+    considerably. Use the `backsize` argument of the open function to
+    make sure some data are loaded into the buffer before the requested
+    frame. Then a subsequent access to the data within backsize `seconds`
+    before that frame can still be handled without the need to reread
+    the file from the beginning.
 
     Usage
     -----
+    With context management:
     ```
     import audioio as aio
     with aio.AudioLoader(filepath, 60.0, 10.0) as data:
@@ -474,6 +532,22 @@ class AudioLoader(object):
     with data.open_audioread(filepath, 60.0, 10.0):
         # do something ...
     ```
+
+    Use `blocks()` for sequential, blockwise reading and processing:
+    ```
+    from scipy.signal import spectrogram
+    nfft = 2048
+    with aio.AudioLoader('some/audio.wav') as data:
+        for x in data.blocks(100*nfft, nfft//2):
+            f, t, Sxx = spectrogram(x, nperseg=nfft, noverlap=nfft//2)
+    ```
+
+    For loop iterates over single frames (1-D arrays containing samples for each channel):
+    ```
+    with aio.AudioLoader('some/audio.wav') as data:
+        for x in data:
+            print(x)
+    ```
     
     Traditional open and close:
     ```
@@ -486,20 +560,9 @@ class AudioLoader(object):
     ```
     data = aio.AudioLoader()
     data.open(filepath, 60.0)
+    ...
     ```
     
-    or for a specific module, here the SoundFile module:
-    ```
-    data = aio.AudioLoader()
-    data.open_soundfile(filepath, 60.0)
-    ```
-    
-    See output of
-    ```
-    aio.list_modules()
-    ```
-    for supported and available modules.
-    
     Parameters
     ----------
     filepath: string
@@ -530,6 +593,10 @@ class AudioLoader(object):
         Open an audio file by trying available audio modules.
     open_*()
         Open an audio file with the respective audio module.
+    __getitem__
+        Access data of the audio file.
+    blocks()
+        Generator for blockwise processing of AudioLoader data.
     close()
         Close the file.
 
@@ -545,12 +612,6 @@ class AudioLoader(object):
     """
     
     def __init__(self, filepath=None, buffersize=10.0, backsize=0.0, verbose=0):
-        """Initialize the AudioLoader instance.
-
-        If filepath is not None open the file.
-
-        See documentation of the class for more information.
-        """
         self.sf = None
         self.samplerate = 0.0
         self.channels = 0
@@ -597,6 +658,7 @@ class AudioLoader(object):
         return self.__next__()
 
     def __getitem__(self, key):
+        """Access data of the audio file."""
         if type(key) is tuple:
             index = key[0]
         else:
@@ -622,18 +684,21 @@ class AudioLoader(object):
             if step is None:
                 step = 1
             else:
-                start = int(start)
+                step = int(step)
             self._update_buffer(start, stop)
             newindex = slice(start-self.offset, stop-self.offset, step)
         elif hasattr(index, '__len__'):
+            index = [inx if inx >= 0 else inx+len(self) for inx in index]
             start = min(index)
             stop = max(index)
-            self._update_buffer(start, stop)
+            self._update_buffer(start, stop+1)
             newindex = [inx-self.offset for inx in index]
         else:
             if index > self.frames:
                 raise IndexError
             index = int(index)
+            if index < 0:
+                index += len(self)
             self._update_buffer(index, index+1)
             newindex = index-self.offset
         if type(key) is tuple:
@@ -641,10 +706,81 @@ class AudioLoader(object):
             return self.buffer[newkey]
         else:
             return self.buffer[newindex]
-        return 0
+
+    def blocks(self, block_size, noverlap=0):
+        """Generator for blockwise processing of AudioLoader data.
+
+        Parameters
+        ----------
+        block_size: int
+            Len of data blocks to be returned.
+        noverlap: int
+            Number of indices successive data points should overlap.
+
+        Yields
+        ------
+        data: array
+            Successive slices of the data managed by AudioLoader.
+
+        Raises
+        ------
+        ValueError
+            `noverlap` larger or equal to `block_size`.
+
+        Examples
+        --------
+        Use it for processing long audio data, like computing a spectrogram with overlap:
+        ```
+        from scipy.signal import spectrogram
+        from audioio import AudioLoader, blocks
+        nfft = 2048
+        with AudioLoader('some/audio.wav') as data:
+            for x in data.blocks(100*nfft, nfft//2):
+                f, t, Sxx = spectrogram(x, nperseg=nfft, noverlap=nfft//2)
+        ```
+        """
+        return blocks(self, block_size, noverlap)
+
+    def _init_buffer(self):
+        """Allocate a buffer of size zero."""
+        self.buffer = np.empty((0, self.channels))
+
+    def _update_buffer(self, start, stop):
+        """Make sure that the buffer contains data between start and stop.
+
+        Parameters
+        ----------
+        start: int
+            Index of the first frame for the buffer.
+        stop: int
+            Index of the last frame for the buffer.
+        """
+        if start < self.offset or stop > self.offset + self.buffer.shape[0]:
+            offset, size = self._read_indices(start, stop)
+            r_offset, r_size = self._recycle_buffer(offset, size)
+            self.offset = offset
+            # load buffer content from file, this is backend specific:
+            self._load_buffer(r_offset, r_size)
 
     def _read_indices(self, start, stop):
-        """ Compute position and size for next read from file. """ 
+        """Compute position and size for next read from file.
+
+        This takes buffersize and backsize into account.
+
+        Parameters
+        ----------
+        start: int
+            Index of the first requested frame.
+        stop: int
+            Index of the last requested frame.
+
+        Returns
+        -------
+        offset: int
+           Frame index for the first frame in the buffer.
+        size: int
+           Number of frames the buffer should hold.
+        """
         offset = start
         size = stop-start
         if size < self.buffersize:
@@ -660,24 +796,36 @@ class AudioLoader(object):
                 if offset < 0:
                     offset = 0
                     size = self.frames - offset
-        if self.verbose > 1:
+        if self.verbose > 2:
             print('  request %6d frames at %d-%d' % (size, offset, offset+size))
         return offset, size
 
-    def _init_buffer(self):
-        """Allocate a buffer of size zero."""
-        self.buffer = np.empty((0, self.channels))
-
-    def _allocate_buffer(self, size):
-        """Make sure the buffer has the right size."""
-        if size != self.buffer.shape[0]:
-            self.buffer = np.empty((size, self.channels))
-
     def _recycle_buffer(self, offset, size):
-        """Move already existing parts of the buffer to their new position (as
+        """Recycle buffer contents and return indices for data to be loaded from file.
+
+        Move already existing parts of the buffer to their new position (as
         returned by _read_indices() ) and return position and size of
         data chunk that still needs to be loaded from file.
+
+        Parameters
+        ----------
+        offset: int
+           Frame index for the first frame in the buffer.
+        size: int
+           Number of frames the buffer should hold.
+
+        Returns
+        -------
+        r_offset: int
+           First frame to be read from file.
+        r_size: int
+           Number of frames to be read from file.
         """
+        def allocate_buffer(size):
+            """Make sure the buffer has the right size."""
+            if size != self.buffer.shape[0]:
+                self.buffer = np.empty((size, self.channels))
+
         r_offset = offset
         r_size = size
         if ( offset >= self.offset and
@@ -688,11 +836,11 @@ class AudioLoader(object):
                 n = size
             m = self.buffer.shape[0]
             buffer = self.buffer[-i:m-i+n,:]
-            self._allocate_buffer(size)
+            allocate_buffer(size)
             self.buffer[:n,:] = buffer
             r_offset += n
             r_size -= n
-            if self.verbose > 1:
+            if self.verbose > 2:
                 print('  recycle %6d frames from %d-%d of the old %d-sized buffer to the front at %d-%d (%d-%d in buffer)'
                        % (n, self.offset+m-i, self.offset+m-i+n, m, offset, offset+n, 0, n))
         elif ( offset + size > self.offset and
@@ -700,14 +848,14 @@ class AudioLoader(object):
             n = offset + size - self.offset
             m = self.buffer.shape[0]
             buffer = self.buffer[:n,:]
-            self._allocate_buffer(size)
+            allocate_buffer(size)
             self.buffer[-n:,:] = buffer
             r_size -= n
-            if self.verbose > 1:
+            if self.verbose > 2:
                 print('  recycle %6d frames from %d-%d of the old %d-sized buffer to the end at %d-%d (%d-%d in buffer)'
                        % (n, self.offset, self.offset+n, m, offset+size-n, offset+size, size-n, size))
         else:
-            self._allocate_buffer(size)
+            allocate_buffer(size)
         return r_offset, r_size
 
     
@@ -734,7 +882,7 @@ class AudioLoader(object):
             The wave module is not installed
         """
         self.verbose = verbose
-        if self.verbose > 1:
+        if self.verbose > 0:
             print('open_wave(filepath) with filepath=%s' % filepath)
         if not audio_modules['wave']:
             self.samplerate = 0.0
@@ -761,7 +909,7 @@ class AudioLoader(object):
         self._init_buffer()
         self.offset = 0
         self.close = self._close_wave
-        self._update_buffer = self._update_buffer_wave
+        self._load_buffer = self._load_buffer_wave
         # read 1 frame to determine the unit of the position values:
         self.p0 = self.sf.tell()
         self.sf.readframes(1)
@@ -770,32 +918,30 @@ class AudioLoader(object):
         return self
 
     def _close_wave(self):
-        """ Close the audio file using the wave module. """
+        """Close the audio file using the wave module. """
         if self.sf is not None:
             self.sf.close()
             self.sf = None
 
-    def _update_buffer_wave(self, start, stop):
-        """Make sure that the buffer contains the data between
-        start and stop using the wave module.
+    def _load_buffer_wave(self, r_offset, r_size):
+        """Load new data from file using the wave module.
+
+        Parameters
+        ----------
+        r_offset: int
+           First frame to be read from file.
+        r_size: int
+           Number of frames to be read from file.
         """
-        if start < self.offset or stop > self.offset + self.buffer.shape[0]:
-            offset, size = self._read_indices(start, stop)
-            r_offset, r_size = self._recycle_buffer(offset, size)
-            # read buffer:
-            self.sf.setpos(r_offset*self.pfac + self.p0)
-            buffer = self.sf.readframes(r_size)
-            buffer = np.frombuffer(buffer, dtype=self.dtype).reshape((-1, self.channels))
-            if self.dtype[0] == 'u':
-                self.buffer[r_offset-offset:r_offset+r_size-offset,:] = buffer * self.factor - 1.0
-            else:
-                self.buffer[r_offset-offset:r_offset+r_size-offset,:] = buffer * self.factor
-            self.offset = offset
-            if self.verbose > 1:
-                print('  read %6d frames at %d' % (r_size, r_offset))
-            if self.verbose > 0:
-                print('  loaded %d frames from %d up to %d'
-                      % (self.buffer.shape[0], self.offset, self.offset+self.buffer.shape[0]))
+        self.sf.setpos(r_offset*self.pfac + self.p0)
+        buffer = self.sf.readframes(r_size)
+        buffer = np.frombuffer(buffer, dtype=self.dtype).reshape((-1, self.channels))
+        if self.dtype[0] == 'u':
+            self.buffer[r_offset-self.offset:r_offset+r_size-self.offset,:] = buffer * self.factor - 1.0
+        else:
+            self.buffer[r_offset-self.offset:r_offset+r_size-self.offset,:] = buffer * self.factor
+        if self.verbose > 1:
+            print('  loaded %6d frames at %d' % (r_size, r_offset))
         
 
     # ewave interface:        
@@ -819,7 +965,7 @@ class AudioLoader(object):
             The ewave module is not installed.
         """
         self.verbose = verbose
-        if self.verbose > 1:
+        if self.verbose > 0:
             print('open_ewave(filepath) with filepath=%s' % filepath)
         if not audio_modules['ewave']:
             self.samplerate = 0.0
@@ -840,32 +986,33 @@ class AudioLoader(object):
         self._init_buffer()
         self.offset = 0
         self.close = self._close_ewave
-        self._update_buffer = self._update_buffer_ewave
+        self._load_buffer = self._load_buffer_ewave
         return self
 
     def _close_ewave(self):
-        """ Close the audio file using the ewave module. """
+        """Close the audio file using the ewave module. """
         if self.sf is not None:
             del self.sf
             self.sf = None
 
-    def _update_buffer_ewave(self, start, stop):
-        """Make sure that the buffer contains the data between
-        start and stop using the ewave module.
+    def _load_buffer_ewave(self, r_offset, r_size):
+        """Load new data from file using the wave module.
+
+        Parameters
+        ----------
+        r_offset: int
+           First frame to be read from file.
+        r_size: int
+           Number of frames to be read from file.
         """
-        if start < self.offset or stop > self.offset + self.buffer.shape[0]:
-            offset, size = self._read_indices(start, stop)
-            r_offset, r_size = self._recycle_buffer(offset, size)
-            # read buffer:
-            buffer = self.sf.read(frames=r_size, offset=r_offset, memmap='r')
-            buffer = ewave.rescale(buffer, 'float')
-            if len(buffer.shape) == 1:
-                buffer = np.reshape(buffer,(-1, 1))
-            self.buffer[r_offset-offset:r_offset+r_size-offset,:] = buffer
-            self.offset = offset
-            if self.verbose > 0:
-                print('  loaded %d frames from %d up to %d'
-                      % (self.buffer.shape[0], self.offset, self.offset+self.buffer.shape[0]))
+        buffer = self.sf.read(frames=r_size, offset=r_offset, memmap='r')
+        buffer = ewave.rescale(buffer, 'float')
+        if len(buffer.shape) == 1:
+            buffer = np.reshape(buffer,(-1, 1))
+        self.buffer[r_offset-self.offset:r_offset+r_size-self.offset,:] = buffer
+        if self.verbose > 1:
+            print('  loaded %d frames from %d up to %d'
+                  % (self.buffer.shape[0], self.offset, self.offset+self.buffer.shape[0]))
 
             
     # soundfile interface:        
@@ -889,7 +1036,7 @@ class AudioLoader(object):
             The SoundFile module is not installed
         """
         self.verbose = verbose
-        if self.verbose > 1:
+        if self.verbose > 0:
             print('open_soundfile(filepath) with filepath=%s' % filepath)
         if not audio_modules['soundfile']:
             self.samplerate = 0.0
@@ -914,28 +1061,30 @@ class AudioLoader(object):
         self._init_buffer()
         self.offset = 0
         self.close = self._close_soundfile
-        self._update_buffer = self._update_buffer_soundfile
+        self._load_buffer = self._load_buffer_soundfile
         return self
 
     def _close_soundfile(self):
-        """ Close the audio file using the SoundFile module. """
+        """Close the audio file using the SoundFile module. """
         if self.sf is not None:
             self.sf.close()
             self.sf = None
 
-    def _update_buffer_soundfile(self, start, stop):
-        """Make sure that the buffer contains the data between
-        start and stop using the SoundFile module.
+    def _load_buffer_soundfile(self, r_offset, r_size):
+        """Load new data from file using the wave module.
+
+        Parameters
+        ----------
+        r_offset: int
+           First frame to be read from file.
+        r_size: int
+           Number of frames to be read from file.
         """
-        if start < self.offset or stop > self.offset + self.buffer.shape[0]:
-            offset, size = self._read_indices(start, stop)
-            r_offset, r_size = self._recycle_buffer(offset, size)
-            self.sf.seek(r_offset, soundfile.SEEK_SET)
-            self.buffer[r_offset-offset:r_offset+r_size-offset,:] = self.sf.read(r_size, always_2d=True)
-            self.offset = offset
-            if self.verbose > 0:
-                print('  loaded %d frames from %d up to %d'
-                      % (self.buffer.shape[0], self.offset, self.offset+self.buffer.shape[0]))
+        self.sf.seek(r_offset, soundfile.SEEK_SET)
+        self.buffer[r_offset-self.offset:r_offset+r_size-self.offset,:] = self.sf.read(r_size, always_2d=True)
+        if self.verbose > 1:
+            print('  loaded %d frames from %d up to %d'
+                  % (self.buffer.shape[0], self.offset, self.offset+self.buffer.shape[0]))
 
             
     # wavefile interface:        
@@ -959,7 +1108,7 @@ class AudioLoader(object):
             The wavefile module is not installed
         """
         self.verbose = verbose
-        if self.verbose > 1:
+        if self.verbose > 0:
             print('open_wavefile(filepath) with filepath=%s' % filepath)
         if not audio_modules['wavefile']:
             self.samplerate = 0.0
@@ -980,30 +1129,32 @@ class AudioLoader(object):
         self._init_buffer()
         self.offset = 0
         self.close = self._close_wavefile
-        self._update_buffer = self._update_buffer_wavefile
+        self._load_buffer = self._load_buffer_wavefile
         return self
 
     def _close_wavefile(self):
-        """ Close the audio file using the wavefile module. """
+        """Close the audio file using the wavefile module. """
         if self.sf is not None:
             self.sf.close()
             self.sf = None
 
-    def _update_buffer_wavefile(self, start, stop):
-        """Make sure that the buffer contains the data between
-        start and stop using the wavefile module.
+    def _load_buffer_wavefile(self, r_offset, r_size):
+        """Load new data from file using the wave module.
+
+        Parameters
+        ----------
+        r_offset: int
+           First frame to be read from file.
+        r_size: int
+           Number of frames to be read from file.
         """
-        if start < self.offset or stop > self.offset + self.buffer.shape[0]:
-            offset, size = self._read_indices(start, stop)
-            r_offset, r_size = self._recycle_buffer(offset, size)
-            self.sf.seek(r_offset, wavefile.Seek.SET)
-            buffer = self.sf.buffer(r_size, dtype=self.buffer.dtype)
-            self.sf.read(buffer)
-            self.buffer[r_offset-offset:r_offset+r_size-offset,:] = buffer.T
-            self.offset = offset
-            if self.verbose > 0:
-                print('  loaded %d frames from %d up to %d'
-                      % (self.buffer.shape[0], self.offset, self.offset+self.buffer.shape[0]))
+        self.sf.seek(r_offset, wavefile.Seek.SET)
+        buffer = self.sf.buffer(r_size, dtype=self.buffer.dtype)
+        self.sf.read(buffer)
+        self.buffer[r_offset-self.offset:r_offset+r_size-self.offset,:] = buffer.T
+        if self.verbose > 1:
+            print('  loaded %d frames from %d up to %d'
+                  % (self.buffer.shape[0], self.offset, self.offset+self.buffer.shape[0]))
 
             
     # audioread interface:        
@@ -1030,7 +1181,7 @@ class AudioLoader(object):
             The audioread module is not installed
         """
         self.verbose = verbose
-        if self.verbose > 1:
+        if self.verbose > 0:
             print('open_audio_read(filepath) with filepath=%s' % filepath)
         if not audio_modules['audioread']:
             self.samplerate = 0.0
@@ -1053,106 +1204,117 @@ class AudioLoader(object):
         self.read_buffer = np.zeros((0,0))
         self.read_offset = 0
         self.close = self._close_audioread
-        self._update_buffer = self._update_buffer_audioread
+        self._load_buffer = self._load_buffer_audioread
         self.filepath = filepath
         self.sf_iter = self.sf.__iter__()
         return self
 
     def _close_audioread(self):
-        """ Close the audio file using the audioread module. """
+        """Close the audio file using the audioread module. """
         if self.sf is not None:
             self.sf.__exit__(None, None, None)
             self.sf = None
 
-    def _update_buffer_audioread(self, start, stop):
-        """Make sure that the buffer contains the data between
-        start and stop using the audioread module.
+    def _load_buffer_audioread(self, r_offset, r_size):
+        """Load new data from file using the wave module.
 
-        audioread can only iterate through a file once.
+        audioread can only iterate through a file once and in blocksizes that are
+        given by audioread. Therefore we keep yet another buffer: `self.read_buffer`
+        at file offset `self.read_offset` containing whatever audioread returned.
+
+        Parameters
+        ----------
+        r_offset: int
+           First frame to be read from file.
+        r_size: int
+           Number of frames to be read from file.
         """
-        if start < self.offset or stop > self.offset + self.buffer.shape[0]:
-            offset, size = self._read_indices(start, stop)
-            r_offset, r_size = self._recycle_buffer(offset, size)
-            # recycle file data:
-            if ( self.read_offset + self.read_buffer.shape[0] >= r_offset + r_size
-                 and self.read_offset < r_offset + r_size ):
-                n = r_offset + r_size - self.read_offset
-                self.buffer[self.read_offset-offset:self.read_offset-offset+n,:] = self.read_buffer[:n,:] / (2.0**15-1.0)
-                if self.verbose > 1:
-                    print('  recycle %6d frames from the front of the read buffer to %d-%d (%d-%d in buffer)'
-                           % (n, self.read_offset, self.read_offset+n, self.read_offset-offset, self.read_offset-offset+n))
-                r_size -= n
-            # go back to beginning of file:
-            if r_offset < self.read_offset:
-                if self.verbose > 1:
-                    print('  rewind')
-                self._close_audioread()
-                self.sf = audioread.audio_open(self.filepath)
-                self.sf_iter = self.sf.__iter__()
+        if ( self.read_offset + self.read_buffer.shape[0] >= r_offset + r_size
+             and self.read_offset < r_offset + r_size ):
+            # read_buffer overlaps at the end of the requested interval:
+            i = 0
+            n = r_offset + r_size - self.read_offset
+            if n > r_size:
+                i += n - r_size
+                n = r_size
+            self.buffer[self.read_offset+i-self.offset:self.read_offset+i-self.offset+n,:] = self.read_buffer[i:i+n,:] / (2.0**15-1.0)
+            if self.verbose > 2:
+                print('  recycle %6d frames from the front of the read buffer at %d-%d (%d-%d in buffer)'
+                       % (n, self.read_offset, self.read_offset+n, self.read_offset-self.offset, self.read_offset-self.offset+n))
+            r_size -= n
+            if r_size <= 0:
+                return
+        # go back to beginning of file:
+        if r_offset < self.read_offset:
+            if self.verbose > 2:
+                print('  rewind')
+            self._close_audioread()
+            self.sf = audioread.audio_open(self.filepath)
+            self.sf_iter = self.sf.__iter__()
+            self.read_buffer = np.zeros((0,0))
+            self.read_offset = 0
+        # read to position:
+        while self.read_offset + self.read_buffer.shape[0] < r_offset:
+            self.read_offset += self.read_buffer.shape[0]
+            try:
+                if hasattr(self.sf_iter, 'next'):
+                    buffer = self.sf_iter.next()
+                else:
+                    buffer = next(self.sf_iter)
+            except StopIteration:
                 self.read_buffer = np.zeros((0,0))
-                self.read_offset = 0
-            # read to position:
-            while self.read_offset + self.read_buffer.shape[0] < r_offset:
-                self.read_offset += self.read_buffer.shape[0]
-                try:
-                    if hasattr(self.sf_iter, 'next'):
-                        buffer = self.sf_iter.next()
-                    else:
-                        buffer = next(self.sf_iter)
-                except StopIteration:
-                    self.read_buffer = np.zeros((0,0))
-                    self.buffer[r_offset-offset:,:] = 0.0
-                    if self.verbose > 1:
-                        print('  caught StopIteration, padded buffer with %d zeros' % r_size)
-                    break
-                self.read_buffer = np.frombuffer(buffer, dtype='<i2').reshape(-1, self.channels)
-                if self.verbose > 2:
-                    print('  read forward by %d frames' % self.read_buffer.shape[0])
-            # recycle file data:
-            if ( self.read_offset + self.read_buffer.shape[0] > r_offset
-                 and self.read_offset <= r_offset ):
-                n = self.read_offset + self.read_buffer.shape[0] - r_offset
-                if n > r_size:
-                    n = r_size
-                self.buffer[r_offset - offset:r_offset - offset + n,:] = self.read_buffer[-n:,:] / (2.0**15-1.0)
+                self.buffer[r_offset-self.offset:,:] = 0.0
                 if self.verbose > 1:
-                    print('  recycle %6d frames from the end of the read buffer at %d-%d to %d-%d (%d-%d in buffer)'
-                           % (n, self.read_offset, self.read_offset + self.read_buffer.shape[0],
-                              r_offset, r_offset+n, r_offset-offset, r_offset+n-offset))
+                    print('  caught StopIteration, padded buffer with %d zeros' % r_size)
+                break
+            self.read_buffer = np.frombuffer(buffer, dtype='<i2').reshape(-1, self.channels)
+            if self.verbose > 2:
+                print('  read forward by %d frames' % self.read_buffer.shape[0])
+        # recycle file data:
+        if ( self.read_offset + self.read_buffer.shape[0] > r_offset
+             and self.read_offset <= r_offset ):
+            i = r_offset - self.read_offset
+            n = self.read_offset + self.read_buffer.shape[0] - r_offset
+            if n > r_size:
+                n = r_size
+            self.buffer[r_offset - self.offset:r_offset - self.offset + n,:] = self.read_buffer[i:i+n,:] / (2.0**15-1.0)
+            if self.verbose > 2:
+                print('  recycle %6d frames from the end of the read buffer at %d-%d to %d-%d (%d-%d in buffer)'
+                       % (n, self.read_offset, self.read_offset + self.read_buffer.shape[0],
+                          r_offset, r_offset+n, r_offset-self.offset, r_offset+n-self.offset))
+            r_offset += n
+            r_size -= n
+        # read data:
+        if self.verbose > 2 and r_size > 0:
+            print('  read    %6d frames at %d-%d (%d-%d in buffer)'
+                   % (r_size, r_offset, r_offset+r_size, r_offset-self.offset, r_offset+r_size-self.offset))
+        while r_size > 0:
+            self.read_offset += self.read_buffer.shape[0]
+            try:
+                if hasattr(self.sf_iter, 'next'):
+                    buffer = self.sf_iter.next()
+                else:
+                    buffer = next(self.sf_iter)
+            except StopIteration:
+                self.read_buffer = np.zeros((0,0))
+                self.buffer[r_offset-self.offset:,:] = 0.0
+                if self.verbose > 1:
+                    print('  caught StopIteration, padded buffer with %d zeros' % r_size)
+                break
+            self.read_buffer = np.frombuffer(buffer, dtype='<i2').reshape(-1, self.channels)
+            n = self.read_buffer.shape[0]
+            if n > r_size:
+                n = r_size
+            if n > 0:
+                self.buffer[r_offset-self.offset:r_offset-self.offset+n,:] = self.read_buffer[:n,:] / (2.0**15-1.0)
+                if self.verbose > 2:
+                    print('    read  %6d frames to %d-%d (%d-%d in buffer)'
+                          % (n, r_offset, r_offset+n, r_offset-self.offset, r_offset+n-self.offset))
                 r_offset += n
                 r_size -= n
-            # read data:
-            if self.verbose > 1 and r_size > 0:
-                print('  read    %6d frames at %d-%d (%d-%d in buffer)'
-                       % (r_size, r_offset, r_offset+r_size, r_offset-offset, r_offset+r_size-offset))
-            while r_size > 0:
-                self.read_offset += self.read_buffer.shape[0]
-                try:
-                    if hasattr(self.sf_iter, 'next'):
-                        buffer = self.sf_iter.next()
-                    else:
-                        buffer = next(self.sf_iter)
-                except StopIteration:
-                    self.read_buffer = np.zeros((0,0))
-                    self.buffer[r_offset-offset:,:] = 0.0
-                    if self.verbose > 1:
-                        print('  caught StopIteration, padded buffer with %d zeros' % r_size)
-                    break
-                self.read_buffer = np.frombuffer(buffer, dtype='<i2').reshape(-1, self.channels)
-                n = self.read_buffer.shape[0]
-                if n > r_size:
-                    n = r_size
-                if n > 0:
-                    self.buffer[r_offset-offset:r_offset+n-offset,:] = self.read_buffer[:n,:] / (2.0**15-1.0)
-                    if self.verbose > 2:
-                        print('    read  %6d frames to %d-%d (%d-%d in buffer)'
-                              % (n, r_offset, r_offset+n, r_offset-offset, r_offset+n-offset))
-                    r_offset += n
-                    r_size -= n
-            self.offset = offset
-            if self.verbose > 0:
-                print('  loaded  %d frames at %d-%d'
-                      % (self.buffer.shape[0], self.offset, self.offset+self.buffer.shape[0]))
+        if self.verbose > 1:
+            print('  loaded  %d frames at %d-%d'
+                  % (self.buffer.shape[0], self.offset, self.offset+self.buffer.shape[0]))
 
                                 
     def open(self, filepath, buffersize=10.0, backsize=0.0, verbose=0):
@@ -1185,8 +1347,7 @@ class AudioLoader(object):
         if len(filepath) == 0:
             raise ValueError('input argument filepath is empty string!')
         if not os.path.isfile(filepath):
-            FileNotFoundException = getattr(__builtins__,'FileNotFoundError', IOError)
-            raise FileNotFoundException('file "%s" not found' % filepath)
+            raise FileNotFoundError('file "%s" not found' % filepath)
         if os.path.getsize(filepath) <= 0:
             raise EOFError('file "%s" is empty (size=0)!' % filepath)
         # list of implemented open functions:
@@ -1208,7 +1369,7 @@ class AudioLoader(object):
                 not_installed.append(lib)
                 continue
             try:
-                open_file(filepath, buffersize, backsize, verbose)
+                open_file(filepath, buffersize, backsize, verbose-1)
                 if self.frames > 0:
                     success = True
                     if verbose > 0:
@@ -1230,7 +1391,7 @@ class AudioLoader(object):
 
     
 def demo(file_path, plot):
-    """ Demo of the audioloader functions.
+    """Demo of the audioloader functions.
 
     Parameters
     ----------
@@ -1299,7 +1460,7 @@ def demo(file_path, plot):
 
 
 def main(args):
-    """ Call demo with command line arguments.
+    """Call demo with command line arguments.
 
     Parameters
     ----------
