@@ -1,6 +1,18 @@
 """
 Query and control installation status and availability of audio modules.
 
+- `installed_modules()`: installed audio modules of a specific function.
+- `available_modules()`: installed and enabled audio modules.
+- `unavailable_modules()`: audio modules that are not installed and not enabled.
+- `disable_module()`: disable audio modules so that they are not used by the audioio functions and classes.
+- `enable_module()`: enable audio modules provided they are installed.
+- `select_module()`: select (enable) a single audio module and disable all others.
+- `list_modules()`: print list of all supported modules and their installation status.
+- `missing_modules()`: missing audio modules that are recommended to be installed.
+- `missing_modules_instructions()`: print installation instructions for missing but useful audio modules.
+- `installation_instruction()`: instructions on how to install a specific audio module.
+- `main()`: command line program for listing installation status of audio modules.
+
 `list_modules()` and `installed_modules()` let you query which audio
 modules are currently installed on your system.
 
@@ -60,8 +72,8 @@ There is no need to install additional audio packages.
 For instructions on specific packages, run `audiomodules` with the name of
 the package supplied as argument:
 
-```
-audiomodules soundfile
+```sh
+> audiomodules soundfile
 ```
 This produces something like this:
 ```text
@@ -70,7 +82,7 @@ Installation instructions for the soundfile module:
 ---------------------------------------------------
 The soundfile package is a wrapper of the sndfile library,
 that supports many different audio file formats.
-See http://pysoundfile.readthedocs.org for a documentation of the soundfile package
+See http://pysoundfile.readthedocs.org for a documentation of the soundfile python wrapper
 and http://www.mega-nerd.com/libsndfile for details on the sndfile library.
 
 First, install the following packages:
@@ -88,7 +100,7 @@ sudo apt-get install python3-soundfile
 """
 
 import sys
-from os.path import exists
+import os
 from .version import __version__, __year__
 
 
@@ -105,17 +117,24 @@ audio_infos = {}
 Keys are the module names, values are the informations. """
 
 audio_instructions_linux = {}
-""" Dictionary with installation instructions for windows.
+""" Dictionary with general installation instructions for linux that
+are needed in addition to installing some packages.
 Keys are the module names, values are the instructions. """
 
 audio_instructions_windows = {}
-""" Dictionary with installation instructions for windows.
+""" Dictionary with general installation instructions for windows that
+are needed in addition to installing some packages.
 Keys are the module names, values are the instructions. """
 
 audio_pip_packages = {}
 """ Dictionary with pip package names of the audio modules.
 Keys are the module names, values are pip package names. """
 
+audio_conda_packages = {}
+""" Dictionary with conda package names of the audio modules.
+Keys are the module names, values are conda package names,
+optionally with a channel specification. """
+
 audio_deb_packages = {}
 """ Dictionary with Linux DEB packages of the audio modules.
 Keys are the module names, values are the package names. """
@@ -124,6 +143,10 @@ audio_rpm_packages = {}
 """ Dictionary with Linux RPM packages of the audio modules.
 Keys are the module names, values are the package names. """
 
+audio_brew_packages = {}
+""" Dictionary with macOS homebrew packages of the audio modules.
+Keys are the module names, values are the package (formulae) names. """
+
 audio_required_deb_packages = {}
 """ Dictionary with Linux DEB packages required for the audio modules.
 Keys are the module names, values are lists of string with package names. """
@@ -132,6 +155,10 @@ audio_required_rpm_packages = {}
 """ Dictionary with Linux RPM packages required for the audio modules.
 Keys are the module names, values are lists of string with package names. """
 
+audio_required_brew_packages = {}
+""" Dictionary with macOS homebrew packages required for the audio modules.
+Keys are the module names, values are lists of string with package (formulae) names. """
+
 audio_fileio = []
 """ List of audio modules used for reading and writing audio files. """
 
@@ -159,6 +186,7 @@ try:
 except ImportError:
     audio_modules['ewave'] = False
 audio_pip_packages['ewave'] = 'ewave'
+audio_conda_packages['ewave'] = '-c auto ewave'
 audio_infos['ewave'] = """The ewave package supports more types of WAV-files than the standard wave module.
 For documentation see https://github.com/melizalab/py-ewave"""
 
@@ -170,8 +198,10 @@ try:
 except ImportError:
     audio_modules['scipy.io.wavfile'] = False
 audio_pip_packages['scipy.io.wavfile'] = 'scipy'
+audio_conda_packages['scipy.io.wavfile'] = 'scipy'
 audio_deb_packages['scipy.io.wavfile'] = 'python3-scipy'
 audio_rpm_packages['scipy.io.wavfile'] = 'python3-scipy'
+audio_brew_packages['scipy.io.wavfile'] = 'scipy'
 audio_infos['scipy.io.wavfile'] = """The scipy package provides very basic functions for reading WAV files.
 For documentation see http://docs.scipy.org/doc/scipy/reference/io.html"""
 
@@ -183,12 +213,13 @@ try:
 except ImportError:
     audio_modules['soundfile'] = False
 audio_pip_packages['soundfile'] = 'SoundFile'
+audio_conda_packages['soundfile'] = '-c conda-forge pysoundfile'
 audio_deb_packages['soundfile'] = 'python3-soundfile'
 audio_required_deb_packages['soundfile'] = ['libsndfile1', 'libsndfile1-dev', 'libffi-dev']
 audio_required_rpm_packages['soundfile'] = ['libsndfile', 'libsndfile-devel', 'libffi-devel']
 audio_infos['soundfile'] = """The soundfile package is a wrapper of the sndfile library,
 that supports many different audio file formats.
-See http://pysoundfile.readthedocs.org for a documentation of the soundfile package
+See http://pysoundfile.readthedocs.org for a documentation of the soundfile python wrapper
 and http://www.mega-nerd.com/libsndfile for details on the sndfile library."""
 
 audio_fileio.append('wavefile')
@@ -201,9 +232,10 @@ except ImportError:
 audio_pip_packages['wavefile'] = 'wavefile'
 audio_required_deb_packages['wavefile'] = ['libsndfile1', 'libsndfile1-dev', 'libffi-dev']
 audio_required_rpm_packages['wavefile'] = ['libsndfile', 'libsndfile-devel', 'libffi-devel']
+audio_required_brew_packages['wavefile'] = ['libsndfile', 'libffi']
 audio_infos['wavefile'] = """The wavefile package is a wrapper of the sndfile library,
 that supports many different audio file formats.
-See https://github.com/vokimon/python-wavefile for documentation of the wavefile package
+See https://github.com/vokimon/python-wavefile for documentation of the wavefile python wrapper
 and http://www.mega-nerd.com/libsndfile for details on the sndfile library."""
         
 audio_fileio.append('audioread')
@@ -214,13 +246,34 @@ try:
 except ImportError:
     audio_modules['audioread'] = False
 audio_pip_packages['audioread'] = 'audioread'
+audio_conda_packages['audioread'] = '-c conda-forge audioread'
 audio_deb_packages['audioread'] = 'python3-audioread'
 audio_rpm_packages['audioread'] = 'python3-audioread'
-audio_required_deb_packages['audioread'] = ['libav-tools']
-audio_required_rpm_packages['audioread'] = ['ffmpeg', 'ffmpeg-devel']
-audio_infos['audioread'] = """The audioread package uses ffmpeg and friends to make mp3 files readable.
+audio_required_deb_packages['audioread'] = ['libav-tools', 'libavcodec-extra']
+audio_required_rpm_packages['audioread'] = ['ffmpeg', 'ffmpeg-devel', 'libavcodec-extra']
+audio_required_brew_packages['audioread'] = ['libav --with-libvorbis --with-sdl --with-theora', 'ffmpeg --with-libvorbis --with-sdl2 --with-theora']
+audio_infos['audioread'] = """The audioread package uses libav (https://libav.org/) or ffmpeg (https://ffmpeg.org/) to make mpeg files readable.
+Install this package for reading mpeg files.
 For documentation see https://github.com/beetbox/audioread"""
         
+audio_fileio.append('pydub')
+try:
+    import pydub
+    audio_modules['pydub'] = True
+    audio_installed.append('pydub')
+except ImportError:
+    audio_modules['pydub'] = False
+audio_pip_packages['pydub'] = 'pydub'
+audio_conda_packages['pydub'] = '-c conda-forge pydub'
+audio_deb_packages['pydub'] = 'python3-pydub'
+audio_rpm_packages['pydub'] = 'python3-pydub'
+audio_required_deb_packages['pydub'] = ['libav-tools', 'libavcodec-extra']
+audio_required_rpm_packages['pydub'] = ['ffmpeg', 'ffmpeg-devel', 'libavcodec-extra']
+audio_required_brew_packages['pydub'] = ['libav --with-libvorbis --with-sdl --with-theora', 'ffmpeg --with-libvorbis --with-sdl2 --with-theora']
+audio_infos['pydub'] = """The pydub package uses libav (https://libav.org/) or ffmpeg (https://ffmpeg.org/) to make mpeg files readable and writeable.
+Install this package if you need to write mpeg files.
+For documentation see https://github.com/jiaaro/pydub"""
+        
 audio_device.append('pyaudio')
 try:
     import pyaudio
@@ -229,10 +282,12 @@ try:
 except ImportError:
     audio_modules['pyaudio'] = False
 audio_pip_packages['pyaudio'] = 'PyAudio'
+audio_conda_packages['pyaudio'] = 'pyaudio'
 audio_deb_packages['pyaudio'] = 'python3-pyaudio'
 audio_rpm_packages['pyaudio'] = 'python3-pyaudio'
 audio_required_deb_packages['pyaudio'] = ['libportaudio2', 'portaudio19-dev']
 audio_required_rpm_packages['pyaudio'] = ['libportaudio', 'portaudio-devel']
+audio_required_brew_packages['pyaudio'] = ['portaudio']
 audio_infos['pyaudio'] = """The pyaudio package is a wrapper of the portaudio library (http://www.portaudio.com).
 For documentation see https://people.csail.mit.edu/hubert/pyaudio"""
 audio_instructions_windows['pyaudio'] = """Download an appropriate (latest version, 32 or 64 bit) wheel from
@@ -251,8 +306,10 @@ try:
 except ImportError:
     audio_modules['sounddevice'] = False
 audio_pip_packages['sounddevice'] = 'sounddevice'
+audio_conda_packages['sounddevice'] = '-c conda-forge python-sounddevice'
 audio_required_deb_packages['sounddevice'] = ['libportaudio2', 'portaudio19-dev', 'python3-cffi']
 audio_required_rpm_packages['sounddevice'] = ['libportaudio', 'portaudio-devel', 'python3-cffi']
+audio_required_brew_packages['sounddevice'] = ['portaudio']
 audio_infos['sounddevice'] = """The sounddevice package is a wrapper of the portaudio library (http://www.portaudio.com). 
 If you have trouble with pyaudio, try this as an alternative.
 For documentation see https://python-sounddevice.readthedocs.io"""
@@ -300,8 +357,7 @@ https://mail.python.org/pipermail/tutor/2012-September/091529.html"""
 
 
 def installed_modules(func='all'):
-    """
-    Installed audio modules of a specific function.
+    """Installed audio modules of a specific function.
 
     By default all installed modules are available. With
     `disable_module()`, `enable_module()` and `select_module()`
@@ -332,8 +388,7 @@ def installed_modules(func='all'):
 
 
 def available_modules(func='all'):
-    """
-    Installed and enabled audio modules.
+    """Installed and enabled audio modules.
 
     By default all installed modules are available. With
     `disable_module()`, `enable_module()` and `select_module()`
@@ -360,8 +415,7 @@ def available_modules(func='all'):
 
 
 def unavailable_modules(func='all'):
-    """
-    Audio modules that are not installed and not enabled.
+    """Audio modules that are not installed and not enabled.
 
     Parameters
     ----------
@@ -385,8 +439,7 @@ def unavailable_modules(func='all'):
 
 
 def disable_module(module=None):
-    """
-    Disable audio modules so that they are not used by the audioio functions and classes.
+    """Disable audio modules so that they are not used by the audioio functions and classes.
 
     To disable all modules except one, call `select_module()`.
     
@@ -408,8 +461,7 @@ def disable_module(module=None):
 
 
 def enable_module(module=None):
-    """
-    Enable audio modules provided they are installed.
+    """Enable audio modules provided they are installed.
     
     Parameters
     ----------
@@ -429,8 +481,7 @@ def enable_module(module=None):
 
 
 def select_module(module):
-    """
-    Select (enable) a single audio module and disable all others.
+    """Select (enable) a single audio module and disable all others.
 
     Undo by calling `enable_module()` without arguments.
     
@@ -518,8 +569,7 @@ def list_modules(module='all', availability=True):
 
 
 def missing_modules(func='all'):
-    """
-    Missing audio modules that are recommended to be installed.
+    """Missing audio modules that are recommended to be installed.
 
     Parameters
     ----------
@@ -540,14 +590,13 @@ def missing_modules(func='all'):
             mods.append('soundfile')
         if 'audioread' not in audio_installed:
             mods.append('audioread')
+        if 'pydub' not in audio_installed:
+            mods.append('pydub')
     if func in ['all', 'device']:
         if 'pyaudio' not in audio_installed and \
            'sounddevice' not in audio_installed and \
            'simpleaudio' not in audio_installed:
-            if sys.platform[0:3] == "win":
-                mods.append('simpleaudio')
-            else:
-                mods.append('sounddevice')
+            mods.append('simpleaudio')
     return mods
 
 
@@ -574,7 +623,7 @@ def missing_modules_instructions(func='all'):
 
 
 def installation_instruction(module):
-    """ Instructions on how to install a specific audio module.
+    """Instructions on how to install a specific audio module.
 
     Parameters
     ----------
@@ -588,9 +637,11 @@ def installation_instruction(module):
     """
     install_package_deb = "sudo apt-get install"
     install_package_rpm = "dnf install"
+    install_package_brew = "brew install"
     install_package = None
     package = None
     required_packages = None
+    multiline = False
     instruction = None
         
     install_pip_deb = "sudo pip install"
@@ -599,9 +650,11 @@ def installation_instruction(module):
     install_pip_win = "pip install"
     install_pip = install_pip_deb
 
+    install_conda = "conda install"
+
     # check operating system:
     if sys.platform[0:5] == "linux":
-        if exists('/etc/redhat-release') or exists('/etc/fedora-release'):
+        if os.path.exists('/etc/redhat-release') or os.path.exists('/etc/fedora-release'):
             install_package = install_package_rpm
             install_pip = install_pip_rpm
             package = audio_rpm_packages.get(module, None)
@@ -612,21 +665,33 @@ def installation_instruction(module):
             required_packages = audio_required_deb_packages.get(module, None)
         instruction = audio_instructions_linux.get(module, None)
     elif sys.platform == "darwin":
-        install_package = ''
+        install_package = install_package_brew
         install_pip = install_pip_osx
+        package = audio_brew_packages.get(module, None)
+        required_packages = audio_required_brew_packages.get(module, None)
+        multiline = True
     elif sys.platform[0:3] == "win":
         install_package = ''
         install_pip = install_pip_win
         instruction = audio_instructions_windows.get(module, None)
+    # check conda:
+    conda = "CONDA_DEFAULT_ENV" in os.environ
+    conda_package = audio_conda_packages.get(module, None)
+    if conda:
+        install_pip = install_pip.replace('sudo ', '')
 
     pip_package = audio_pip_packages.get(module, None)
         
     req_inst = None
     if required_packages is not None:
-        if pip_package is None and package is None:
-            req_inst = 'Install the following packages:\n\n%s %s' % (install_package, ' '.join(required_packages))
+        if multiline:
+            ps = '\n'.join([install_package + ' ' + p for p in required_packages])
         else:
-            req_inst = 'First, install the following packages:\n\n%s %s' % (install_package, ' '.join(required_packages))
+            ps = install_package + ' ' + ' '.join(required_packages)
+        if pip_package is None and package is None:
+            req_inst = 'Install the following packages:\n\n' + ps
+        else:
+            req_inst = 'First, install the following packages:\n\n' + ps
     
     pip_inst = None
     if pip_package is not None:
@@ -635,14 +700,19 @@ def installation_instruction(module):
     dist_inst = None
     if package is not None:
         if pip_inst is None:
-            dist_inst = 'Install module from your distributions package:\n\n%s %s' % (install_package, package)
+            dist_inst = 'Install module from your distribution\'s package:\n\n%s %s' % (install_package, package)
         else:
             dist_inst = 'or alternatively from your distribution\'s package:\n\n%s %s' % (install_package, package)
 
+    conda_inst = None
+    if conda and conda_package is not None:
+        conda_inst = 'Install the %s module with conda:\n\n%s %s' % (module, install_conda, conda_package)
+        req_inst = pip_inst = dist_inst = instruction = None
+
     info = audio_infos.get(module, None)
 
     msg = ''
-    for s in [info, req_inst, pip_inst, dist_inst, instruction]:
+    for s in [info, conda_inst, req_inst, pip_inst, dist_inst, instruction]:
         if s is not None:
             if len(msg) > 0:
                 msg += '\n\n'
@@ -651,7 +721,7 @@ def installation_instruction(module):
     return msg
 
 
-def main(args):
+def main(args=None):
     """ Command line program for listing installation status of audio modules.
 
     Run this module as a script
@@ -659,14 +729,14 @@ def main(args):
     > python -m audioio.auidomodules
     ```
     or, when the audioio package is installed on your system, simply run
-    ```
+    ```sh
     > audiomodules
     ```
     for an overview of audio packages, their installation status, and recommendations on
     how to install further audio packages.
 
     The '--help' argument prints out a help message:
-    ```
+    ```sh
     > audiomodules --help
     ```
 
@@ -675,6 +745,8 @@ def main(args):
     args: list of strings
         Command line arguments as provided by sys.argv
     """
+    if args is None:
+        args = sys.argv
     if len(args) > 1 :
         if args[1] == '--version':
             print('version', __version__, 'by Benda-Lab (2015-%s)' % __year__)
@@ -712,4 +784,4 @@ def main(args):
 
 
 if __name__ == "__main__":
-    main(sys.argv)
+    main()
