@@ -4,6 +4,7 @@ Writing numpy arrays of floats to audio files.
 - `write_audio()`: write audio data to file.
 - `available_formats()`: audio file formats supported by any of the installed audio modules.
 - `available_encodings()`: encodings of an audio file format supported by any of the installed audio modules.
+- `format_from_extension()`: deduce audio file format from file extension.
 
 The data to be written are 1-D or 2-D numpy arrays of floats ranging between -1 and 1
 with first axis time and second axis channel.
@@ -17,11 +18,42 @@ For a demo, run the script as:
 python -m audioio.audiowriter
 ```
 """
- 
+
+import os
+import subprocess
 import numpy as np
 from .audiomodules import *
 
 
+def format_from_extension(filepath):
+    """Deduce audio file format from file extension.
+
+    Parameters
+    ----------
+    filepath: string
+        Name of the audio file.
+
+    Returns
+    -------
+    format: string
+        Audio format deduced from file extension.
+    """
+    if not filepath:
+        return None
+    ext = os.path.splitext(filepath)[1]
+    if not ext:
+        return None
+    if ext[0] == '.':
+        ext = ext[1:]
+    if not ext:
+        return None
+    ext = ext.upper()
+    if ext == 'WAVE':
+        return 'WAV'
+    ext = ext.replace('MPEG' , 'MP')
+    return ext
+
+
 def formats_wave():
     """Audio file formats supported by the wave module.
 
@@ -58,8 +90,7 @@ def encodings_wave(format):
 
 
 def write_wave(filepath, data, samplerate, format=None, encoding=None):
-    """
-    Write audio data using the wave module from pythons standard libray.
+    """Write audio data using the wave module from pythons standard libray.
     
     Documentation
     -------------
@@ -91,19 +122,21 @@ def write_wave(filepath, data, samplerate, format=None, encoding=None):
     """
     if not audio_modules['wave']:
         raise ImportError
+    if not filepath:
+        raise ValueError('no file specified!')
 
+    if not format:
+        format = format_from_extension(filepath)
     if format and format.upper() != 'WAV':
         raise ValueError('file format %s not supported by wave module' % format)
 
     wave_encodings = {'PCM_32': [4, 'i4'],
                       'PCM_16': [2, 'i2'],
                       'PCM_U8': [1, 'u1'] }
-    if encoding is None:
-        encoding = ''
-    if len(encoding) == 0:
+    if not encoding:
         encoding = 'PCM_16'
     encoding = encoding.upper()
-    if not encoding in wave_encodings:
+    if encoding not in wave_encodings:
         raise ValueError('file encoding %s not supported by wave module' % encoding)
     sampwidth = wave_encodings[encoding][0]
     dtype = wave_encodings[encoding][1]
@@ -166,8 +199,7 @@ def encodings_ewave(format):
 
 
 def write_ewave(filepath, data, samplerate, format=None, encoding=None):
-    """
-    Write audio data using the ewave module from pythons standard libray.
+    """Write audio data using the ewave module from pythons standard libray.
 
     Documentation
     -------------
@@ -199,7 +231,11 @@ def write_ewave(filepath, data, samplerate, format=None, encoding=None):
     """
     if not audio_modules['ewave']:
         raise ImportError
+    if not filepath:
+        raise ValueError('no file specified!')
 
+    if not format:
+        format = format_from_extension(filepath)
     if format and format.upper() != 'WAV' and format.upper() != 'WAVEX':
         raise ValueError('file format %s not supported by ewave module' % format)
 
@@ -208,12 +244,10 @@ def write_ewave(filepath, data, samplerate, format=None, encoding=None):
                        'PCM_16': 'h',
                        'FLOAT': 'f',
                        'DOUBLE': 'd' }
-    if encoding == '':
-        encoding = None
-    if encoding is None:
+    if not encoding:
         encoding = 'PCM_16'
     encoding = encoding.upper()
-    if not encoding in ewave_encodings:
+    if encoding not in ewave_encodings:
         raise ValueError('file encoding %s not supported by ewave module' % encoding)
 
     channels = 1
@@ -261,8 +295,7 @@ def encodings_wavfile(format):
 
 
 def write_wavfile(filepath, data, samplerate, format=None, encoding=None):
-    """
-    Write audio data using the scipy.io.wavfile module.
+    """Write audio data using the scipy.io.wavfile module.
     
     Documentation
     -------------
@@ -294,7 +327,11 @@ def write_wavfile(filepath, data, samplerate, format=None, encoding=None):
     """
     if not audio_modules['scipy.io.wavfile']:
         raise ImportError
+    if not filepath:
+        raise ValueError('no file specified!')
 
+    if not format:
+        format = format_from_extension(filepath)
     if format and format.upper() != 'WAV':
         raise ValueError('file format %s not supported by scipy.io.wavfile module' % format)
 
@@ -304,12 +341,10 @@ def write_wavfile(filepath, data, samplerate, format=None, encoding=None):
                       'PCM_64': [8, 'i8'],
                       'FLOAT': [4, 'f'],
                       'DOUBLE': [8, 'd']}
-    if encoding == '':
-        encoding = None
-    if encoding is None:
+    if not encoding:
         encoding = 'PCM_16'
     encoding = encoding.upper()
-    if not encoding in wave_encodings:
+    if encoding not in wave_encodings:
         raise ValueError('file encoding %s not supported by scipy.io.wavfile module' % encoding)
     sampwidth = wave_encodings[encoding][0]
     dtype = wave_encodings[encoding][1]
@@ -360,8 +395,7 @@ def encodings_soundfile(format):
 
 
 def write_soundfile(filepath, data, samplerate, format=None, encoding=None):
-    """
-    Write audio data using the SoundFile module (based on libsndfile).
+    """Write audio data using the SoundFile module (based on libsndfile).
     
     Documentation
     -------------
@@ -391,18 +425,20 @@ def write_soundfile(filepath, data, samplerate, format=None, encoding=None):
     """
     if not audio_modules['soundfile']:
         raise ImportError
+    if not filepath:
+        raise ValueError('no file specified!')
 
-    if format is not None:
-        if len(format) == 0:
-            format = None
-        else:
-            format = format.upper()
+    if not format:
+        format = format_from_extension(filepath)
+    if not format:
+        format = 'WAV'
+    if format:
+        format = format.upper()
 
-    if encoding == '':
-        encoding = None
-    if encoding is None:
+    if not encoding:
         encoding = 'PCM_16'
     encoding = encoding.upper()
+    
     soundfile.write(filepath, data, int(samplerate), format=format, subtype=encoding)
 
 
@@ -454,8 +490,7 @@ def encodings_wavefile(format):
 
     
 def write_wavefile(filepath, data, samplerate, format=None, encoding=None):
-    """
-    Write audio data using the wavefile module (based on libsndfile).
+    """Write audio data using the wavefile module (based on libsndfile).
     
     Documentation
     -------------
@@ -487,26 +522,24 @@ def write_wavefile(filepath, data, samplerate, format=None, encoding=None):
     """
     if not audio_modules['wavefile']:
         raise ImportError
+    if not filepath:
+        raise ValueError('no file specified!')
 
-    if format is None:
-        format = ''
-    if len(format) == 0:
-        format = filepath.split('.')[-1]
-        # TODO: we need a mapping from file extensions to formats and default encoding!
+    if not format:
+        format = format_from_extension(filepath)
+    if not format:
+        format = 'WAV'
     format = format.upper()
     try:
         format_value = getattr(wavefile.Format, format)
     except AttributeError:
         raise ValueError('file format %s not supported by wavefile module' % format)
 
-    if encoding is None:
-        encoding = ''
-    if len(encoding) == 0:
+    if not encoding:
         encodings = encodings_wavefile(format)
+        encoding = encodings[0]
         if 'PCM_16' in encodings:
             encoding = 'PCM_16'
-        else:
-            encoding = encodings[0]
     encoding = encoding.upper()
     try:
         encoding_value = getattr(wavefile.Format, encoding)
@@ -523,12 +556,149 @@ def write_wavefile(filepath, data, samplerate, format=None, encoding=None):
         w.write(data.T)
 
 
+def formats_pydub():
+    """Audio file formats supported by the Pydub module.
+
+    Returns
+    -------
+    formats: list of strings
+        List of supported file formats as strings.
+    """
+    if not audio_modules['pydub']:
+        return []
+    formats = []
+    command = [pydub.AudioSegment.converter, '-formats']
+    with open(os.devnull, 'rb') as devnull:
+        p = subprocess.Popen(command, stdin=devnull, stdout=subprocess.PIPE,
+                             stderr=subprocess.PIPE, universal_newlines=True)
+        skip = True
+        for line in p.communicate()[0].split('\n'):
+            if '--' in line[:3]:
+                skip = False
+                continue
+            if skip:
+                continue
+            cols = line.split()
+            if len(cols) > 2 and 'E' in cols[0]:
+                formats.append(cols[1].upper())
+    return formats
+
+def encodings_pydub(format):
+    """Encodings of an audio file format supported by the Pydub module.
+
+    Parameters
+    ----------
+    format: str
+        The file format.
+
+    Returns
+    -------
+    encodings: list of strings
+        List of supported encodings as strings.
+    """
+    pydub_encodings = {'pcm_s16le': 'PCM_16',
+                       'pcm_s24le': 'PCM_24',
+                       'pcm_s32le': 'PCM_32',
+                       'pcm_f32le': 'FLOAT',
+                       'pcm_f64le': 'DOUBLE',
+                       }    
+    if not audio_modules['pydub']:
+        return []
+    if format.upper() not in formats_pydub():
+        return []
+    encodings = []
+    command = [pydub.AudioSegment.converter, '-encoders']
+    with open(os.devnull, 'rb') as devnull:
+        p = subprocess.Popen(command, stdin=devnull, stdout=subprocess.PIPE,
+                             stderr=subprocess.PIPE, universal_newlines=True)
+        skip = True
+        for line in p.communicate()[0].split('\n'):
+            if '--' in line[:3]:
+                skip = False
+                continue
+            if skip:
+                continue
+            cols = line.split()
+            if len(cols) > 2 and cols[0][0] == 'A':
+                encoding = cols[1]
+                if encoding in pydub_encodings:
+                    encoding = pydub_encodings[encoding]
+                encodings.append(encoding.upper())
+    return encodings
+
+def write_pydub(filepath, data, samplerate, format=None, encoding=None):
+    """Write audio data using the Pydub module.
+    
+    Documentation
+    -------------
+    https://github.com/jiaaro/pydub
+
+    Parameters
+    ----------
+    filepath: string
+        Full path and name of the file to write.
+    data: 1-D or 2-D array of floats
+        Array with the data (first index time, second index channel,
+        values within -1.0 and 1.0).
+    samplerate: float
+        Sampling rate of the data in Hertz.
+    format: string or None
+        File format, everything ffmpeg or avtools are supporting.
+    encoding: string or None
+        Encoding of the data.
+        If None or empty string use 'PCM_16'.
+
+    Raises
+    ------
+    ImportError
+        The Pydub module is not installed.
+    *
+        Writing of the data failed.
+    ValueError
+        File format or encoding not supported.
+    """
+    if not audio_modules['pydub']:
+        raise ImportError
+    if not filepath:
+        raise ValueError('no file specified!')
+
+    if not format:
+        format = format_from_extension(filepath)
+    if format and format.upper() not in formats_pydub():
+        raise ValueError('file format %s not supported by Pydub module' % format)
+
+    pydub_encodings = {'PCM_16': 'pcm_s16le',
+                       'PCM_24': 'pcm_s24le',
+                       'PCM_32': 'pcm_s32le',
+                       'DOUBLE': 'pcm_f32le',
+                       'FLOAT': 'pcm_f64le',
+                       }    
+    if encoding:
+        encoding = encoding.upper()
+        if encoding in pydub_encodings:
+            encoding = pydub_encodings[encoding]
+        if encoding not in encodings_pydub(format):
+            raise ValueError('file encoding %s not supported by Pydub module' % encoding)
+        encoding = encoding.lower()
+    else:
+        encoding = None
+        
+    channels = 1
+    if len(data.shape) > 1:
+        channels = data.shape[1]
+    int_data = (data*(2**31-1)).astype(np.int32)
+    sound = pydub.AudioSegment(int_data.ravel(), sample_width=4,
+                               frame_rate=samplerate, channels=channels)
+    sound.export(filepath, format=format.lower(), codec=encoding)
+
+
 audio_formats_funcs = (
     ('soundfile', formats_soundfile),
     ('wavefile', formats_wavefile),
     ('wave', formats_wave),
     ('ewave', formats_ewave),
-    ('scipy.io.wavfile', formats_wavfile)
+    ('scipy.io.wavfile', formats_wavfile),
+    ('pydub', formats_pydub)
     )
 """ List of implemented formats functions.
 
@@ -555,7 +725,8 @@ audio_encodings_funcs = (
     ('wavefile', encodings_wavefile),
     ('wave', encodings_wave),
     ('ewave', encodings_ewave),
-    ('scipy.io.wavfile', encodings_wavfile)
+    ('scipy.io.wavfile', encodings_wavfile),
+    ('pydub', encodings_pydub)
     )
 """ List of implemented encodings functions.
 
@@ -593,7 +764,8 @@ audio_writer_funcs = (
     ('wavefile', write_wavefile),
     ('wave', write_wave),
     ('ewave', write_ewave),
-    ('scipy.io.wavfile', write_wavfile)
+    ('scipy.io.wavfile', write_wavfile),
+    ('pydub', write_pydub)
     )
 """ List of implemented write functions.
 
@@ -602,8 +774,7 @@ Each element of the list is a tuple with the module's name and the write functio
 
 
 def write_audio(filepath, data, samplerate, format=None, encoding=None, verbose=0):
-    """
-    Write audio data to file.
+    """Write audio data to file.
 
     Parameters
     ----------
@@ -643,9 +814,8 @@ def write_audio(filepath, data, samplerate, format=None, encoding=None, verbose=
     write_audio('audio/file.wav', data, samplerate)
     ```
     """
-
-    if len(filepath) == 0:
-        raise ValueError('input argument filepath is empty string!')
+    if not filepath:
+        raise ValueError('no file specified!')
 
     # write audio file by trying available modules:
     success = False
@@ -669,8 +839,8 @@ def write_audio(filepath, data, samplerate, format=None, encoding=None, verbose=
         raise IOError('failed to write data to file "%s"' % filepath)
 
 
-def demo(file_path, encoding=''):
-    """ Demo of the audiowriter functions.
+def demo(file_path, channels=2, encoding=''):
+    """Demo of the audiowriter functions.
 
     Parameters
     ----------
@@ -679,11 +849,12 @@ def demo(file_path, encoding=''):
     encoding: string
         Encoding to be used.
     """
-    print('')
     print('generate data ...')
     samplerate = 44100.0
-    t = np.arange(0.0, 2.0, 1.0/samplerate)
-    data = np.sin(2.0*np.pi*880.0*t)
+    t = np.arange(0.0, 1.0, 1.0/samplerate)
+    data = np.zeros((len(t), channels))
+    for c in range(channels):
+        data[:,c] = np.sin(2.0*np.pi*(440.0+c*8.0)*t)
         
     print("write_audio(%s) ..." % file_path)
     write_audio(file_path, data, samplerate, encoding=encoding, verbose=2)
@@ -692,28 +863,33 @@ def demo(file_path, encoding=''):
     
 
 def main(args):
-    """ Call demo with command line arguments.
+    """Call demo with command line arguments.
 
     Parameters
     ----------
     args: list of strings
         Command line arguments as provided by sys.argv
     """
-    print("Checking audiowriter module ...")
-
     help = False
     file_path = None
     encoding = ''
     mod = False
+    nchan = False
+    channels = 2
     for arg in args[1:]:
         if mod:
             select_module(arg)
             mod = False
+        elif nchan:
+            channels = int(arg)
+            nchan = False
         elif arg == '-h':
             help = True
             break
         elif arg == '-m':
             mod = True
+        elif arg == '-n':
+            nchan = True
         elif file_path is None:
             file_path = arg
         else:
@@ -725,10 +901,10 @@ def main(args):
     if help:
         print('')
         print('Usage:')
-        print('  python -m audioio.audiowriter [-m module] [<filename>] [<encoding>]')
+        print('  python -m audioio.audiowriter [-m module] [-n channels] [<filename>] [<encoding>]')
         return
 
-    demo(file_path, encoding)
+    demo(file_path, channels=channels, encoding=encoding)
 
 
 if __name__ == "__main__":
