@@ -124,8 +124,7 @@ def note2freq(note, a4freq=440.0):
 
 
 def fade_in(data, rate, fadetime):
-    """
-    Fade in a signal in place.
+    """Fade in a signal in place.
 
     The first `fadetime` seconds of the data are multiplied with a squared sine in place.
     
@@ -149,8 +148,7 @@ def fade_in(data, rate, fadetime):
 
         
 def fade_out(data, rate, fadetime):
-    """
-    Fade out a signal in place.
+    """Fade out a signal in place.
 
     The last `fadetime` seconds of the data are multiplied with a squared sine in place.
     
@@ -174,8 +172,7 @@ def fade_out(data, rate, fadetime):
 
 
 def fade(data, rate, fadetime):
-    """
-    Fade in and out a signal in place.
+    """Fade in and out a signal in place.
 
     The first and last `fadetime` seconds of the data are multiplied with
     a squared sine in place.
@@ -232,10 +229,6 @@ class PlayAudio(object):
     """
     
     def __init__(self, verbose=0):
-        """Initialize module for playing audio.
-
-        See `PlayAudio` documentation for details.
-        """
         self.verbose = verbose
         self.handle = None
         self._do_play = self._play
@@ -274,7 +267,8 @@ class PlayAudio(object):
 
         Raises
         ------
-        ValueError: Invalid sampling rate (after some attemps of resampling).
+        ValueError
+            Invalid sampling rate (after some attemps of resampling).
         """
         if self.handle is None:
             self.open()
@@ -318,7 +312,8 @@ class PlayAudio(object):
 
         Raises
         ------
-        ValueError: Invalid sampling rate (after some attemps of resampling).
+        ValueError
+            Invalid sampling rate (after some attemps of resampling).
         
         See also
         --------
@@ -383,7 +378,10 @@ class PlayAudio(object):
 
         Raises
         ------
-        ImportError: pyaudio module is not available.            
+        ImportError
+            PyAudio module is not available.
+        OSError
+            Cannot open audio device.
 
         Documentation
         -------------
@@ -416,15 +414,23 @@ class PlayAudio(object):
         os.dup(oldstderr)
         os.close(oldstderr)
         os.remove(tmpfile)
+        self.close = self._close_pyaudio
+        self.stop = self._stop_pyaudio
+        try:
+            info = self.handle.get_default_output_device_info()
+            self.max_channels = info['maxOutputChannels']
+            self.default_rate = info['defaultSampleRate']
+            self.device_index = info['index']
+            self.handle.is_format_supported(48000, output_device=self.device_index,
+                                            output_channels=1, output_format=pyaudio.paInt16)
+        except Exception as e:
+            if self.verbose > 0:
+                print(str(e))
+            self.handle.terminate()
+            raise OSError('failed to initialize audio device')
         self.index = 0
         self.data = None
         self._do_play = self._play_pyaudio
-        self.close = self._close_pyaudio
-        self.stop = self._stop_pyaudio
-        info = self.handle.get_default_output_device_info()
-        self.max_channels = info['maxOutputChannels']
-        self.default_rate = info['defaultSampleRate']
-        self.device_index = info['index']
         return self
 
     def _callback_pyaudio(self, in_data, frames, time_info, status):
@@ -488,7 +494,8 @@ class PlayAudio(object):
 
         Raises
         ------
-        ValueError: Invalid sampling rate (after some attemps of resampling).
+        ValueError
+            Invalid sampling rate (after some attemps of resampling).
         """
         # check channel count:
         channels = self.channels
@@ -557,7 +564,10 @@ class PlayAudio(object):
 
         Raises
         ------
-        ImportError: sounddevice module is not available.            
+        ImportError
+           sounddevice module is not available.            
+        OSError
+            Cannot open audio device.
 
         Documentation
         -------------
@@ -575,14 +585,22 @@ class PlayAudio(object):
         self.handle = True
         self.index = 0
         self.data = None
-        self._do_play = self._play_sounddevice
+        self.stream = None
         self.close = self._close_sounddevice
         self.stop = self._stop_sounddevice
-        self.device_index = sounddevice.default.device[1]
-        info = sounddevice.query_devices(self.device_index)
-        self.max_channels = info['max_output_channels']
-        self.default_rate = info['default_samplerate']
-        self.stream = None
+        try:
+            self.device_index = sounddevice.default.device[1]
+            info = sounddevice.query_devices(self.device_index)
+            self.max_channels = info['max_output_channels']
+            self.default_rate = info['default_samplerate']
+            sounddevice.check_output_settings(device=self.device_index,
+                                              channels=1, dtype=np.int16,
+                                              samplerate=48000)
+        except Exception as e:
+            if self.verbose > 0:
+                print(str(e))
+            raise OSError('failed to initialize audio device')
+        self._do_play = self._play_sounddevice
         return self
 
     def _callback_sounddevice(self, out_data, frames, time_info, status):
@@ -655,7 +673,8 @@ class PlayAudio(object):
 
         Raises
         ------
-        ValueError: Invalid sampling rate (after some attemps of resampling).
+        ValueError
+            Invalid sampling rate (after some attemps of resampling).
         """
         # check channel count:
         channels = self.channels
@@ -721,7 +740,8 @@ class PlayAudio(object):
 
         Raises
         ------
-        ImportError: simpleaudio module is not available.
+        ImportError
+            simpleaudio module is not available.
 
         Documentation
         -------------
@@ -741,8 +761,7 @@ class PlayAudio(object):
             self.handle.stop()
     
     def _play_simpleaudio(self, blocking=True):
-        """
-        Play audio data using the simpleaudio package.
+        """Play audio data using the simpleaudio package.
 
         Parameters
         ----------
@@ -751,7 +770,8 @@ class PlayAudio(object):
 
         Raises
         ------
-        ValueError: Invalid sampling rate (after some attemps of resampling).
+        ValueError
+            Invalid sampling rate (after some attemps of resampling).
         """
         rates = [self.rate, 44100, 22050]
         scales = [1, None, None]
@@ -761,6 +781,10 @@ class PlayAudio(object):
                 scale = self.rate/float(rate)
             if scale != 1:
                 self._down_sample(self.channels, scale)
+            oldstderr = os.dup(2)
+            os.close(2)
+            tmpfile = 'tmpfile.tmp'
+            os.open(tmpfile, os.O_WRONLY | os.O_CREAT)
             try:
                 self.handle = simpleaudio.play_buffer(self.data, self.channels,
                                                       2, int(self.rate))
@@ -769,13 +793,22 @@ class PlayAudio(object):
             except ValueError:
                 if self.verbose > 0:
                     print('invalid sampling rate of %g Hz' % rate)
+            except Exception as e:
+                if self.verbose > 0:
+                    print(str(e))
+                return
+            finally:
+                os.close(2)
+                os.dup(oldstderr)
+                os.close(oldstderr)
+                os.remove(tmpfile)
         if not success:
             raise ValueError('No valid sampling rate found')
         elif blocking:
             self.handle.wait_done()
         
     def _close_simpleaudio(self):
-        """Close audio output using simpleaudio package. """
+        """Close audio output using simpleaudio package."""
         self._stop_simpleaudio()
         simpleaudio.stop_all()
         self.handle = None
@@ -790,7 +823,10 @@ class PlayAudio(object):
 
         Raises
         ------
-        ImportError: ossaudiodev module is not available.
+        ImportError
+            ossaudiodev module is not available.
+        OSError
+            Cannot open audio device.
 
         Documentation
         -------------
@@ -806,15 +842,20 @@ class PlayAudio(object):
         """
         if not audio_modules['ossaudiodev']:
             raise ImportError
-        handle = ossaudiodev.open('w')
-        handle.close()
         self.handle = True
+        self.close = self._close_ossaudiodev
+        self.stop = self._stop_ossaudiodev
         self.osshandle = None
         self.run = False
         self.play_thread = None
+        try:
+            handle = ossaudiodev.open('w')
+            handle.close()
+        except Exception as e:
+            if self.verbose > 0:
+                print(str(e))
+            raise OSError('failed to initialize audio device')
         self._do_play = self._play_ossaudiodev
-        self.close = self._close_ossaudiodev
-        self.stop = self._stop_ossaudiodev
         return self
 
     def _stop_ossaudiodev(self):
@@ -839,12 +880,12 @@ class PlayAudio(object):
             self.run = False
         
     def _play_ossaudiodev(self, blocking=True):
-        """
-        Play audio data using the ossaudiodev module.
+        """Play audio data using the ossaudiodev module.
 
         Raises
         ------
-        ValueError: Invalid sampling rate (after some attemps of resampling).
+        ValueError
+            Invalid sampling rate (after some attemps of resampling).
 
         Parameters
         ----------
@@ -892,7 +933,7 @@ class PlayAudio(object):
             self.play_thread.start()
 
     def _close_ossaudiodev(self):
-        """Close audio output using ossaudiodev module. """
+        """Close audio output using ossaudiodev module."""
         self._stop_ossaudiodev()
         self.handle = None
         self._do_play = self._play
@@ -906,7 +947,8 @@ class PlayAudio(object):
 
         Raises
         ------
-        ImportError: winsound module is not available.
+        ImportError
+            winsound module is not available.
 
         Documentation
         -------------
@@ -924,11 +966,13 @@ class PlayAudio(object):
 
     def _stop_winsound(self):
         """Stop any ongoing activity of the winsound module."""
-        winsound.PlaySound(None, winsound.SND_MEMORY)
-    
+        try:
+            winsound.PlaySound(None, winsound.SND_MEMORY)
+        except Exception as e:
+            pass
+        
     def _play_winsound(self, blocking=True):
-        """
-        Play audio data using the winsound module.
+        """Play audio data using the winsound module.
 
         Parameters
         ----------
@@ -949,7 +993,12 @@ class PlayAudio(object):
             except AttributeError:
                 w.writeframes(self.data.tostring())
             w.close()
-            winsound.PlaySound(self.data_buffer.getvalue(), winsound.SND_MEMORY)
+            try:
+                winsound.PlaySound(self.data_buffer.getvalue(), winsound.SND_MEMORY)
+            except Exception as e:
+                if self.verbose > 0:
+                    print(str(e))
+                return
         else:
             if self.verbose > 0:
                 print('Warning: asynchronous playback is limited to playing wav files by the winsound module. Install an alternative package as recommended by the audiomodules script. ')
@@ -965,10 +1014,15 @@ class PlayAudio(object):
             except AttributeError:
                 w.writeframes(self.data.tostring())
             w.close()
-            winsound.PlaySound(self.audio_file, winsound.SND_ASYNC)
+            try:
+                winsound.PlaySound(self.audio_file, winsound.SND_ASYNC)
+            except Exception as e:
+                if self.verbose > 0:
+                    print(str(e))
+                return
         
     def _close_winsound(self):
-        """Close audio output using winsound module. """
+        """Close audio output using winsound module."""
         self._stop_winsound()
         self.handle = None
         if len(self.audio_file) > 0 and os.path.isfile(self.audio_file):
@@ -1040,8 +1094,7 @@ def play(data, rate, scale=None, blocking=True, verbose=0):
     
 def beep(duration=0.5, frequency=880.0, amplitude=0.5, rate=44100.0,
          fadetime=0.05, blocking=True, verbose=0):
-    """
-    Playback a tone.
+    """Playback a tone.
 
     Create an PlayAudio instance on the globale variable handle.
 
@@ -1071,8 +1124,7 @@ def beep(duration=0.5, frequency=880.0, amplitude=0.5, rate=44100.0,
 
 
 def demo():
-    """ Demonstrate the playaudio module.
-    """
+    """ Demonstrate the playaudio module."""
     print('play mono beep 1')
     audio = PlayAudio(verbose=2)
     audio.beep(1.0, 440.0)
@@ -1102,7 +1154,7 @@ def demo():
 
 
 def main(args):
-    """ Call demo with command line arguments.
+    """Call demo with command line arguments.
 
     Parameters
     ----------
